from datetime import timedelta

import fastapi as fa
from fastapi.security import OAuth2PasswordRequestForm

from api.router import router
from db import init_db
import services


def setup():
    app = fa.FastAPI()
    app.include_router(router)

    return app


app = setup()

init_db(app)


@router.post("/token", response_model=services.Token)
async def login_for_access_token(form_data: OAuth2PasswordRequestForm = fa.Depends()):
    user = services.authenticate_user(services.fake_users_db, form_data.username, form_data.password)

    if not user:
        raise fa.HTTPException(
            status_code=fa.status.HTTP_401_UNAUTHORIZED,
            detail="Incorrect username or password",
            headers={"WWW-Authenticate": "Bearer"},
        )

    access_token_expires = timedelta(minutes=services.ACCESS_TOKEN_EXPIRE_MINUTES)

    access_token = services.create_access_token(
        data={"sub": user.username}, expires_delta=access_token_expires
        )

    return {"access_token": access_token, "token_type": "bearer"}


@router.get("/users/me/", response_model=services.User)
async def read_users_me(current_user: services.User = fa.Depends(services.get_current_active_user)):
    return current_user


if __name__ == '__main__':
    import uvicorn

    uvicorn.run('main:app', host='0.0.0.0', port=8000, reload=True)
